SystemOrganization addCategory: #'GsUpgrader-Core'!

Object subclass: #GsUpgrader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GsUpgrader-Core'!

!GsUpgrader commentStamp: '' prior: 0!
for more information see https://github.com/GsDevKit/gsUpgrader#gsupgrader-!

!GsUpgrader class methodsFor: 'analysis' stamp: 'dkh 09/11/2014 16:35'!
metacelloReport
  "produce a report listing the configurations, baselines and packages loaded in the image"

  | strm |
  strm := WriteStream on: String new.
  self metacelloReportOn: strm.
  ^ strm contents! !

!GsUpgrader class methodsFor: 'analysis' stamp: 'dkh 09/11/2014 16:35'!
metacelloReportOn: strm
  "produce a report listing the configurations, baselines and packages loaded in the image"

  self new
    projectReportOn: strm;
    packageReportOn: strm! !

!GsUpgrader class methodsFor: 'projects' stamp: 'dkh 09/11/2014 16:09'!
upgradeGLASS
  "Ensure that your image is upgraded to GLASS1.0-beta.9.3"

  self new upgradeGLASS! !

!GsUpgrader class methodsFor: 'projects' stamp: 'dkh 09/11/2014 16:21'!
upgradeGLASS1
  "Ensure that your image is upgraded to the latest version of GLASS1"

  self new upgradeGLASS1! !

!GsUpgrader class methodsFor: 'projects' stamp: 'dkh 09/11/2014 16:21'!
upgradeGrease
  "Ensure that your image is upgraded to the latest version of Grease"

  self new upgradeGrease! !

!GsUpgrader class methodsFor: 'projects' stamp: 'dkh 09/11/2014 16:21'!
upgradeMetacello
  "Ensure that your image is upgraded to the latest version of Metacello"

  self new upgradeMetacello! !

!GsUpgrader methodsFor: 'reporting' stamp: 'dkh 09/11/2014 17:03'!
configurationVersionReportOn: strm
  "configuration versions"

  | count |
  count := 1.
  strm
    nextPutAll: '------------------------';
    cr;
    nextPutAll: 'Metacello Version REPORT';
    cr;
    nextPutAll: '------------------------';
    cr.
  ((Object allSubclasses
    select: [ :cl | 
      cl isMeta not
        and: [ cl isMetacelloConfig and: [ cl name asString beginsWith: 'ConfigurationOf' ] ] ])
    sort: [ :a :b | a name < b name ])
    collect: [ :each | 
      | cv |
      cv := each project currentVersion.
      cv ~~ nil
        ifTrue: [ 
          strm nextPutAll: count printString , '. '.
          strm
            nextPutAll:
                each name asString , '	' , cv versionNumber printString , '	'
                    , cv blessing printString;
            cr.
          count := count + 1 ] ]! !

!GsUpgrader methodsFor: 'testing' stamp: 'dkh 09/11/2014 09:46'!
glass1Loaded
  ^ self standardBaselineLoaded: 'BaselineOfGLASS1'! !

!GsUpgrader methodsFor: 'testing' stamp: 'dkh 09/05/2014 13:06'!
glass1Present
  Smalltalk at: #'BaselineOfGLASS1' ifPresent: [ :ignored | ^ true ].
  ^ false! !

!GsUpgrader methodsFor: 'private' stamp: 'dkh 09/05/2014 13:04'!
glass1RepositoryDescription
  ^ 'github://glassdb/glass:master/repository'! !

!GsUpgrader methodsFor: 'testing' stamp: 'dkh 09/12/2014 10:45'!
greaseBaselineLoaded
  ^ self standardBaselineLoaded: 'BaselineOfGrease'! !

!GsUpgrader methodsFor: 'testing' stamp: 'dkh 09/10/2014 15:01'!
greaseLocked
  (self metacelloClass image
    baseline: [ :spec | spec name = 'Grease' ];
    list)
    do: [ :greaseSpec | 
      greaseSpec isLocked
        ifTrue: [ ^ true ] ].
  ^ false! !

!GsUpgrader methodsFor: 'private' stamp: 'dkh 09/11/2014 09:35'!
greaseRepositoryDescription
  ^ 'github://GsDevKit/Grease:master/repository'! !

!GsUpgrader methodsFor: 'private' stamp: 'dkh 09/05/2014 16:56'!
inform: message
  self log: message.
  super inform: message! !

!GsUpgrader methodsFor: 'loading' stamp: 'dkh 09/10/2014 15:01'!
loadGLASS1
  | metacello |
  self upgradeMetacello.
  metacello := self metacelloClass new
    baseline: 'GLASS1';
    repository: self glass1RepositoryDescription;
    yourself.
  GsDeployer
    deploy: [ 
      self log: 'Installing GLASS1'.
      metacello copy get.
      metacello copy load ]! !

!GsUpgrader methodsFor: 'loading' stamp: 'dkh 09/12/2014 10:48'!
loadGreaseBaseline
  | metacello |
  self upgradeMetacello.
  metacello := self metacelloClass new
    baseline: 'Grease';
    repository: self greaseRepositoryDescription;
    yourself.
  GsDeployer
    deploy: [ 
      self log: 'Installing Grease'.
      metacello copy get.
      metacello copy load ]! !

!GsUpgrader methodsFor: 'loading' stamp: 'dkh 09/05/2014 16:54'!
loadMetacelloPreview
  "Bootstrap Metacello Preview, using mcz files (#'previewBootstrap' symbolic version"

  self metacelloPreviewLoaded
    ifTrue: [ 
      self log: 'Metacello Preview already loaded'.
      ^ self ].
  self log: 'Loading Metacello Preview'.
  Gofer new
    gemsource: 'metacello';
    package: 'ConfigurationOfMetacello';
    load.
  GsDeployer
    deploy: [ 
      "load the Metacello Preview"
      ((Smalltalk at: #'ConfigurationOfMetacello') project
        version: #'previewBootstrap') load.
      (Smalltalk at: #'Metacello') new
        configuration: 'MetacelloPreview';
        version: #'stable';
        repository: self metacelloPreviewRepositoryDescription;
        load ]! !

!GsUpgrader methodsFor: 'locking' stamp: 'dkh 09/11/2014 09:47'!
lockGrease
  self greaseLocked
    ifTrue: [ ^ self ].
  self log: 'Locking Grease'.
  GsDeployer
    deploy: [ 
      self metacelloClass new
        baseline: 'Grease';
        repository: self greaseRepositoryDescription;
        lock ]! !

!GsUpgrader methodsFor: 'private' stamp: 'dkh 09/05/2014 16:53'!
log: message
  Transcript
    cr;
    show: '======================';
    cr;
    show: '=====' , message! !

!GsUpgrader methodsFor: 'private' stamp: 'dkh 09/10/2014 15:01'!
metacelloClass
  ^ Smalltalk at: #'Metacello'! !

!GsUpgrader methodsFor: 'testing' stamp: 'dkh 09/11/2014 09:46'!
metacelloPreviewLoaded
  ^ self standardBaselineLoaded: 'BaselineOfMetacello'! !

!GsUpgrader methodsFor: 'private' stamp: 'dkh 09/05/2014 12:23'!
metacelloPreviewRepositoryDescription
  ^ 'github://dalehenrich/metacello-work:configuration'! !

!GsUpgrader methodsFor: 'reporting' stamp: 'dkh 09/12/2014 10:24'!
metacelloRegistrationReportOn: strm
  | map |
  strm
    nextPutAll: '-----------------------------';
    cr;
    nextPutAll: 'Metacello Registration REPORT';
    cr;
    nextPutAll: '-----------------------------';
    cr.
  map := Dictionary new.
  MetacelloProjectRegistration registry projectSpecs
    do: [ :projectSpec | 
      MetacelloProjectRegistration
        registrationForProjectSpec: projectSpec
        ifAbsent: [ self error: 'registration for projectSpec: ' projectSpec name , ' not found' ]
        ifPresent: [ :registration :ignored | map at: registration baseName put: registration ] ].
  (map values sorted: [ :a :b | a baseName < b baseName ])
    do: [ :registration | 
      | isLoaded isLocked |
      isLocked := registration locked.
      isLoaded := registration loadedInImage.
      isLoaded
        ifFalse: [ strm nextPutAll: '<' ].
      isLocked
        ifTrue: [ strm nextPutAll: '|' ].
      strm nextPutAll: registration baseName.
      isLocked
        ifTrue: [ strm nextPutAll: '|' ].
      strm space.
      (registration respondsTo: #'currentVersionString')
        ifTrue: [ 
          strm
            nextPutAll:
              registration currentVersionString , ' [' , registration currentBranchName , ']' ]
        ifFalse: [ strm nextPutAll: registration projectSpec versionString ].
      isLoaded
        ifFalse: [ strm nextPutAll: '>' ].
      strm cr ]! !

!GsUpgrader methodsFor: 'private' stamp: 'dkh 09/05/2014 12:23'!
metacelloRepositoryDescription
  ^ 'github://dalehenrich/metacello-work:master/repository'! !

!GsUpgrader methodsFor: 'private' stamp: 'dkh 09/05/2014 16:56'!
notify: message
  self log: message.
  super notify: message! !

!GsUpgrader methodsFor: 'reporting' stamp: 'dkh 09/11/2014 17:02'!
packageReportOn: strm
  strm
    nextPutAll: '--------------';
    cr;
    nextPutAll: 'PACKAGE REPORT';
    cr;
    nextPutAll: '--------------';
    cr.
  (MCWorkingCopy allManagers sorted: [ :a :b | a packageName < b packageName ])
    do: [ :wc | 
      strm
        nextPutAll: wc ancestry ancestorString;
        cr ]! !

!GsUpgrader methodsFor: 'reporting' stamp: 'dkh 09/11/2014 16:38'!
projectReportOn: strm
  "Use registraion and get configuration/baseline versions otherwise get configuration versions"

  self metacelloPreviewLoaded
    ifTrue: [ self metacelloRegistrationReportOn: strm ]
    ifFalse: [ self configurationVersionReportOn: strm ]! !

!GsUpgrader methodsFor: 'private' stamp: 'dkh 09/11/2014 09:45'!
standardBaselineLoaded: baselineClassName
  Smalltalk
    at: baselineClassName
    ifPresent: [ :ignored | 
      Smalltalk
        at: #'MetacelloProjectRegistration'
        ifPresent: [ :cls | 
          (cls registrationForClassNamed: baselineClassName ifAbsent: [  ])
            ifNotNil: [ :registration | ^ registration loadedInImage ] ] ].
  ^ false! !

!GsUpgrader methodsFor: 'private' stamp: 'dkh 09/11/2014 09:38'!
standardUpgrade: projectName repository: repositoryDescription
  | projectSpec metacello |
  projectSpec := (self metacelloClass image
    baseline: [ :spec | spec name = projectName ];
    list) first.
  projectSpec isLocked
    ifTrue: [ 
      projectSpec repositorySpecs
        detect: [ :repoSpec | repoSpec description = repositoryDescription ]
        ifNone: [ 
          self
            notify:
              projectName , ' project repository is not in standard location ('
                , self metacelloRepositoryDescription , '). The upgrade of '
                , projectName , ' may not be complete.' ].
      metacello := self metacelloClass image baseline: projectName ]
    ifFalse: [ 
      metacello := self metacelloClass new
        baseline: projectName;
        repository: repositoryDescription;
        yourself ].
  GsDeployer
    deploy: [ 
      self log: 'Upgrading ' , projectName.
      metacello copy get.
      metacello copy load ]! !

!GsUpgrader methodsFor: 'upgrading' stamp: 'dkh 09/10/2014 16:09'!
upgradeGLASS
  "skip this step if GLASS1 is already loaded"

  | glassVersion |
  self glass1Loaded
    ifTrue: [ 
      self inform: 'GLASS1 is loaded. Skipping GLASS upgrade.'.
      ^ self ].
  glassVersion := ConfigurationOfGLASS project currentVersion.
  self metacelloPreviewLoaded
    ifTrue: [ 
      self inform: 'MetacelloPreview is loaded. Skipping GLASS upgrade.'.
      ^ self ].
  glassVersion versionNumber < '1.0-beta.9.3' asMetacelloVersionNumber
    ifTrue: [ 
      self log: 'Upgrading GLASS to 1.0-beta.9.3'.
      GsDeployer
        deploy: [ 
          Gofer new
            package: 'ConfigurationOfGLASS';
            url: 'http://seaside.gemtalksystems.com/ss/MetacelloRepository';
            load.
          glassVersion versionNumber < '1.0-beta.9.2' asMetacelloVersionNumber
            ifTrue: [ 
              [ 
              ((Smalltalk at: #'ConfigurationOfGLASS') project version: '1.0-beta.9.2')
                load: #('Core' 'Monticello' 'Gofer Project Loader').
              ((Smalltalk at: #'ConfigurationOfGLASS') project
                version: '1.0-beta.9.2') load ]
                on: MetacelloSkipDirtyPackageLoad
                do: [ :ex | ex resume: false ] ].
          ((Smalltalk at: #'ConfigurationOfGLASS') project
            version: '1.0-beta.9.3') load ] ]
    ifFalse: [ self log: 'GLASS already upgraded to 1.0-beta.9.3' ]! !

!GsUpgrader methodsFor: 'upgrading' stamp: 'dkh 09/11/2014 09:48'!
upgradeGLASS1
  | metacello projectSpec |
  self glass1Loaded
    ifFalse: [ 
      self upgradeGLASS.
      self glass1Present
        ifFalse: [ 
          ^ self
            loadGLASS1;
            upgradeGrease ] ].
  self
    upgradeMetacello;
    upgradeGrease;
    standardUpgrade: 'GLASS1' repository: self glass1RepositoryDescription! !

!GsUpgrader methodsFor: 'upgrading' stamp: 'dkh 09/12/2014 10:48'!
upgradeGrease
  self greaseBaselineLoaded
    ifTrue: [ 
      self
        upgradeMetacello;
        standardUpgrade: 'Grease' repository: self greaseRepositoryDescription ]
    ifFalse: [ self loadGreaseBaseline ].
  self lockGrease! !

!GsUpgrader methodsFor: 'upgrading' stamp: 'dkh 09/11/2014 09:42'!
upgradeMetacello
  self
    loadMetacelloPreview;
    standardUpgrade: 'Metacello'
      repository: self metacelloRepositoryDescription! !
